'From Squeak3.7alpha of ''15 October 2003'' [latest update: #5566] on 30 November 2003 at 12:31:04 pm'!
"Change Set:		B3DTutorial
Date:			30 November 2003
Author:			Boris Gaertner

Due to a change in B3DIndexedMesh, the computation of vertex normals (which are needed to color a scene with lights) are now computed in a different way. The the methods in this change file adapt the tutorial to that change.
Stephane Rollandin most friendly told my about that problem."!


!B3DDemoSurfaces methodsFor: 'common methods' stamp: 'BG 11/30/2003 12:21'!
createQuadGeometry: fn 

  " here we create a QuadMesh. The scene is created elsewhere. "

   | nroPx nroPy stepPx stepPy x y
     low high xStart xStop yStart yStop idxF
     vtx face faces  mesh |

  nroPx := 48.  " number of points in direction x "
  nroPy := 48.  " number of points in direction y "
  xStart := 0.0.
  xStop  := 6.25.
  yStart := 0.0.
  yStop  := 6.25.
  stepPx := xStop - xStart / (nroPx - 1).
  stepPy := yStop - yStart / (nroPy - 1).
  
  vtx _ WriteStream on: (B3DVector3Array new: nroPx * nroPy).
  x := xStart.
  y := yStart.
    " compute the vertices "
  nroPx timesRepeat:
    [y := yStart.
     nroPy timesRepeat:
       [vtx nextPut: (B3DVector3 x: x y: (fn value: x value: y) z: y).
        y := y + stepPy.  
       ].
      x := x + stepPx. 
    ].

   "  create the quads "
   faces _ B3DIndexedQuadArray new: (nroPx - 1)*(nroPy - 1).
   idxF := low := 1.
	1 to: nroPx -1 do: [:i |
         high := low + nroPy.
		1 to: nroPy - 1 do: [:j |
			face _ B3DIndexedQuad 
						with: low + j - 1
						with: low + j
						with: high + j
						with: high + j - 1.
			faces at: idxF put: face.
              idxF := idxF + 1.
		].
        low := high.
      ].


   mesh := B3DIndexedQuadMesh new.
   mesh vertices: vtx contents;
          faces: faces.
  mesh vertexNormals: mesh computeVertexNormals.
  ^mesh
  ! !

!B3DDemoSurfaces methodsFor: 'common methods' stamp: 'BG 11/30/2003 12:19'!
createQuadGeometryAndTexture: fn 

  " here we create a QuadMesh. The scene is created elsewhere. "

   | nroPx nroPy stepPx stepPy x y xTexOrigin yTexOrigin
     low high xStart xStop yStart yStop idxF
     vtx face faces tex mesh |

  nroPx := 48.  " number of points in direction x "
  nroPy := 48.  " number of points in direction y "
  xStart := 0.0.
  xStop  := 6.25.
  yStart := 0.0.
  yStop  := 6.25.
  stepPx := xStop - xStart / (nroPx - 1).
  stepPy := yStop - yStart / (nroPy - 1).

 
   vtx := WriteStream on: (B3DVector3Array new: nroPx * nroPy).
  x := xStart.
  y := yStart.
    " compute the vertices "
  nroPx timesRepeat:
    [y := yStart.
     nroPy timesRepeat:
       [vtx nextPut: (B3DVector3 x: x y: (fn value: x value: y) z: y).
        y := y + stepPy.  
       ].
      x := x + stepPx. 
    ].

    " define the texture "
     yTexOrigin := 0.0.
    tex _ WriteStream on: (B3DTexture2Array new: nroPx * nroPy).
 
   1 to: nroPx do:[:xx | xTexOrigin := 0.0.
        1 to: nroPy do: [:idx | 
               tex nextPut:  xTexOrigin @ yTexOrigin.
               xTexOrigin := xTexOrigin + 0.5.
		].
      yTexOrigin := yTexOrigin + 0.5
   ].
   "  create the quads "
   faces _ B3DIndexedQuadArray new: (nroPx - 1)*(nroPy - 1).
   idxF := low := 1.
	1 to: nroPx -1 do: [:i |
         high := low + nroPy.
		1 to: nroPy - 1 do: [:j |
			face _ B3DIndexedQuad 
						with: low + j - 1
						with: low + j
						with: high + j
						with: high + j - 1.
			faces at: idxF put: face.
              idxF := idxF + 1.
		].
        low := high.
      ].


   mesh := B3DIndexedQuadMesh new.
   mesh vertices: vtx contents;
          texCoords: tex contents;
           faces: faces.
 
  mesh vertexNormals: mesh computeVertexNormals.
  ^mesh
  ! !

!B3DDemoSurfaces methodsFor: 'common methods' stamp: 'BG 11/30/2003 12:18'!
createTriangleGeometryAndTexture: fn 

  " here we create a QuadMesh. The scene is created elsewhere. "

   | nroPx nroPy stepPx stepPy x y xTexOrigin yTexOrigin
     low high xStart xStop yStart yStop idxF
     vtx face faces tex mesh |

  nroPx := 48.  " number of points in direction x "
  nroPy := 48.  " number of points in direction y "
  xStart := 0.0.
  xStop  := 6.25.
  yStart := 0.0.
  yStop  := 6.25.
  stepPx := xStop - xStart / (nroPx - 1).
  stepPy := yStop - yStart / (nroPy - 1).
  
  vtx _ WriteStream on: (B3DVector3Array new: nroPx * nroPy).
  x := xStart.
  y := yStart.
    " compute the vertices "
  nroPx timesRepeat:
    [y := yStart.
     nroPy timesRepeat:
       [vtx nextPut: (B3DVector3 x: x y: (fn value: x value: y) z: y).
        y := y + stepPy.  
       ].
      x := x + stepPx. 
    ].

    " define the texture "
     yTexOrigin := 0.0.
    tex _ WriteStream on: (B3DTexture2Array new: nroPx * nroPy).
 
   1 to: nroPx do:[:xx | xTexOrigin := 0.0.
        1 to: nroPy do: [:idx | 
               tex nextPut:  xTexOrigin @ yTexOrigin.
               xTexOrigin := xTexOrigin + 0.5.
		].
      yTexOrigin := yTexOrigin + 0.5
    ].
   "  create the triangles "
     faces := B3DIndexedTriangleArray new: (nroPx - 1)*(nroPy - 1)*2.  

   idxF := low := 1.
   1 to: nroPx -1 do: [:i |
      high := low + nroPy.
      1 to: nroPy - 1 do: [:j |
         face := B3DIndexedTriangle 
                   with: low + j - 1
                   with: low + j
                   with: high + j.
         faces at: idxF put: face.
         idxF := idxF + 1.
         face := B3DIndexedTriangle 
                   with: low + j - 1
                   with: high + j
                   with: high + j - 1.
         faces at: idxF put: face.
         idxF := idxF + 1.
        ].
      low := high.
     ].


   mesh := B3DIndexedTriangleMesh  new.
   mesh vertices: vtx contents;
          texCoords: tex contents;
        " vertexNormals: vtxNormals; "
          faces: faces.
  mesh vertexNormals: mesh computeVertexNormals.
  ^mesh
  ! !

!B3DDemoSurfaces methodsFor: 'demo5' stamp: 'BG 11/30/2003 12:21'!
computeGeometry3x: aBlockX y: aBlockY z: aBlockZ uFrom: u1 to: u2 vFrom: v1 to: v2
divisionU: gridU divisionV: gridV

   "  compute the mesh for a surface that is defined by three functions of two parameters. "
 

	| vtx face dU dV vertices faces  mesh tex xTexOrigin yTexOrigin low high |
   
    dU := u2 - u1 / (gridU - 1).  " gridU, gridV is the number of points "
    dV := v2 - v1 / (gridV - 1).
	vtx _ WriteStream on: (B3DVector3Array new": gridU * gridV").
  	0 to: gridU - 1 do:[:idU | | u | 
                 u := dU * idU + u1.
		0 to: gridV - 1 do:[:idV| | v x y fn |
               v := dV * idV + v1.
               x := aBlockX value: u value: v.
               y := aBlockY value: u value: v.
               fn := aBlockZ value: u value: v.
			vtx nextPut: (B3DVector3 x: x asFloat  y: y asFloat  z: fn asFloat ).
 		].
	].

     " define the texture "
       yTexOrigin := 0.0.
     tex _ WriteStream on: (B3DTexture2Array new).
   	0 to: gridU - 1 do:[:x| xTexOrigin := 0.0.
        0 to: gridV - 1 do: [:idx| 
               tex nextPut:  xTexOrigin @ yTexOrigin.
               xTexOrigin := xTexOrigin + 0.5.
		].
      yTexOrigin := yTexOrigin + 0.5
   ].

	vertices _ vtx contents.
   
	faces _ WriteStream on: (B3DIndexedQuadArray new).
     low := 1.
	1 to: gridU -1 do:[:i|
         high := low + gridV.
		1 to: gridV - 1 do:[:j|
			face _ B3DIndexedQuad 
						with: low + j - 1
						with: low + j
						with: high + j
						with: high + j - 1.
			faces nextPut: face.
		].
        low := high.
 ].
   mesh := B3DIndexedQuadMesh new.
   mesh vertices: vertices;
          faces: faces contents;
          texCoords: tex contents.
  mesh vertexNormals: mesh computeVertexNormals.
  ^mesh! !

